@@ -3,11 +3,11 @@ use std::{path::Path, sync::Arc};
 use datasize::DataSize;
 use serde::Deserialize;
 
+use casper_hashing::Digest;
 use casper_types::{ProtocolVersion, PublicKey, SecretKey};
 
 use crate::{
     components::consensus::{protocols::highway::config::Config as HighwayConfig, EraId},
-    crypto::hash::Digest,
     types::{chainspec::HighwayConfig as HighwayProtocolConfig, Chainspec, TimeDiff, Timestamp},
     utils::{External, LoadError, Loadable},
 };
@@ -16,11 +16,11 @@ use crate::{
 #[derive(DataSize, Debug, Deserialize, Clone)]
 // Disallow unknown fields to ensure config files and command-line overrides contain valid keys.
 #[serde(deny_unknown_fields)]
-pub struct Config {
+pub(crate) struct Config {
     /// Path to secret key file.
-    pub secret_key_path: External<Arc<SecretKey>>,
+    pub(crate) secret_key_path: External,
     /// Highway-specific node configuration.
-    pub highway: HighwayConfig,
+    pub(crate) highway: HighwayConfig,
 }
 
 impl Default for Config {
@@ -39,8 +39,8 @@ impl Config {
         &self,
         root: P,
     ) -> Result<(Arc<SecretKey>, PublicKey), LoadError<<Arc<SecretKey> as Loadable>::Error>> {
-        let secret_signing_key = self.secret_key_path.clone().load(root)?;
-        let public_key = PublicKey::from(secret_signing_key.as_ref());
+        let secret_signing_key: Arc<SecretKey> = self.secret_key_path.clone().load(root)?;
+        let public_key: PublicKey = PublicKey::from(secret_signing_key.as_ref());
         Ok((secret_signing_key, public_key))
     }
 }
@@ -48,7 +48,7 @@ impl Config {
 /// Consensus protocol configuration.
 #[derive(DataSize, Debug)]
 pub(crate) struct ProtocolConfig {
-    pub(crate) highway_config: HighwayProtocolConfig,
+    pub(crate) highway: HighwayProtocolConfig,
     pub(crate) era_duration: TimeDiff,
     pub(crate) minimum_era_height: u64,
     /// Number of eras before an auction actually defines the set of validators.
@@ -72,7 +72,7 @@ pub(crate) struct ProtocolConfig {
 impl From<&Chainspec> for ProtocolConfig {
     fn from(chainspec: &Chainspec) -> Self {
         ProtocolConfig {
-            highway_config: chainspec.highway_config,
+            highway: chainspec.highway_config,
             era_duration: chainspec.core_config.era_duration,
             minimum_era_height: chainspec.core_config.minimum_era_height,
             auction_delay: chainspec.core_config.auction_delay,
