@@ -5,6 +5,7 @@ mod error;
 mod operations;
 mod types;
 
+use once_cell::sync::Lazy;
 use std::{
     collections::BTreeMap,
     fmt::{self, Debug, Formatter},
@@ -46,12 +47,40 @@ use crate::{
 pub(crate) use announcements::ContractRuntimeAnnouncement;
 pub(crate) use config::Config;
 pub(crate) use error::{BlockExecutionError, ConfigError};
-pub(crate) use types::{BlockAndExecutionEffects, EraValidatorsRequest};
+pub use operations::execute_finalized_block;
+pub use types::BlockAndExecutionEffects;
+pub(crate) use types::EraValidatorsRequest;
+
+/// Maximum number of resource intensive tasks that can be run in parallel.
+///
+/// TODO: Fine tune this constant to the machine executing the node.
+const MAX_PARALLEL_INTENSIVE_TASKS: usize = 4;
+
+/// Semaphore enforcing maximum number of parallel resource intensive tasks.
+static INTENSIVE_TASKS_SEMAPHORE: Lazy<tokio::sync::Semaphore> =
+    Lazy::new(|| tokio::sync::Semaphore::new(MAX_PARALLEL_INTENSIVE_TASKS));
+
+/// Asynchronously runs a resource intensive task.
+/// At most `MAX_PARALLEL_INTENSIVE_TASKS` are being run in parallel at any time.
+///
+/// The task is a closure that takes no arguments and returns a value.
+/// This function returns a future for that value.
+async fn run_intensive_task<T, V>(task: T) -> V
+where
+    T: 'static + Send + FnOnce() -> V,
+    V: 'static + Send,
+{
+    // This will never panic since the semaphore is never closed.
+    let _permit = INTENSIVE_TASKS_SEMAPHORE.acquire().await.unwrap();
+    tokio::task::spawn_blocking(task)
+        .await
+        .expect("task panicked")
+}
 
 /// State to use to construct the next block in the blockchain. Includes the state root hash for the
 /// execution engine as well as certain values the next header will be based on.
 #[derive(DataSize, Debug, Clone, Serialize)]
-pub(crate) struct ExecutionPreState {
+pub struct ExecutionPreState {
     /// The height of the next `Block` to be constructed. Note that this must match the height of
     /// the `FinalizedBlock` used to generate the block.
     next_block_height: u64,
@@ -118,7 +147,7 @@ impl Debug for ContractRuntime {
 
 /// Metrics for the contract runtime component.
 #[derive(Debug)]
-pub(crate) struct ContractRuntimeMetrics {
+pub struct ContractRuntimeMetrics {
     run_execute: Histogram,
     apply_effect: Histogram,
     commit_upgrade: Histogram,
@@ -401,27 +430,31 @@ where
                 transfers,
                 responder,
             } => {
-                let block_height = finalized_block.height();
-                info!(
+                trace!(
                     ?protocol_version,
-                    %block_height,
-                    "executing block"
+                    ?execution_pre_state,
+                    ?finalized_block,
+                    ?deploys,
+                    "execute block request"
                 );
                 let engine_state = Arc::clone(&self.engine_state);
                 let metrics = Arc::clone(&self.metrics);
-                tokio::task::unconstrained(async move {
-                    let result = operations::execute_finalized_block(
-                        engine_state.as_ref(),
-                        metrics.as_ref(),
-                        protocol_version,
-                        execution_pre_state,
-                        finalized_block,
-                        deploys,
-                        transfers,
-                    );
-                    info!(?protocol_version, %block_height, "finished executing block");
+                async move {
+                    let result = run_intensive_task(move || {
+                        execute_finalized_block(
+                            engine_state.as_ref(),
+                            Some(metrics),
+                            protocol_version,
+                            execution_pre_state,
+                            finalized_block,
+                            deploys,
+                            transfers,
+                        )
+                    })
+                    .await;
+                    trace!(?result, "execute block response");
                     responder.respond(result).await
-                })
+                }
                 .ignore()
             }
             ContractRuntimeRequest::EnqueueBlockForExecution {
@@ -483,6 +516,7 @@ where
 }
 
 impl ContractRuntime {
+    #[allow(clippy::too_many_arguments)]
     pub(crate) fn new(
         protocol_version: ProtocolVersion,
         storage_dir: &Path,
@@ -490,6 +524,7 @@ impl ContractRuntime {
         wasm_config: WasmConfig,
         system_config: SystemConfig,
         max_associated_keys: u32,
+        max_runtime_call_stack_height: u32,
         registry: &Registry,
     ) -> Result<Self, ConfigError> {
         // TODO: This is bogus, get rid of this
@@ -517,6 +552,7 @@ impl ContractRuntime {
         let engine_config = EngineConfig::new(
             contract_runtime_config.max_query_depth(),
             max_associated_keys,
+            max_runtime_call_stack_height,
             wasm_config,
             system_config,
         );
@@ -604,10 +640,10 @@ impl ContractRuntime {
             block,
             execution_results,
             maybe_step_effect_and_upcoming_era_validators,
-        } = match tokio::task::unconstrained(async move {
-            operations::execute_finalized_block(
+        } = match run_intensive_task(move || {
+            execute_finalized_block(
                 engine_state.as_ref(),
-                metrics.as_ref(),
+                Some(metrics),
                 protocol_version,
                 current_execution_pre_state,
                 finalized_block,
@@ -629,11 +665,11 @@ impl ContractRuntime {
         announcements::linear_chain_block(effect_builder, block, execution_results).await;
 
         if let Some(StepEffectAndUpcomingEraValidators {
-            step_execution_effect,
+            step_execution_journal,
             upcoming_era_validators,
         }) = maybe_step_effect_and_upcoming_era_validators
         {
-            announcements::step_success(effect_builder, current_era_id, step_execution_effect)
+            announcements::step_success(effect_builder, current_era_id, step_execution_journal)
                 .await;
 
             announcements::upcoming_era_validators(
