@@ -1,7 +1,7 @@
 use std::collections::{BTreeMap, HashMap};
 
-use casper_execution_engine::core::engine_state::{
-    execution_effect::ExecutionEffect, GetEraValidatorsRequest,
+use casper_execution_engine::{
+    core::engine_state::GetEraValidatorsRequest, shared::execution_journal::ExecutionJournal,
 };
 use casper_hashing::Digest;
 use casper_types::{EraId, ExecutionResult, ProtocolVersion, PublicKey, U512};
@@ -84,10 +84,10 @@ impl From<EraValidatorsRequest> for GetEraValidatorsRequest {
 /// Effects from running step and the next era validators that are gathered when an era ends.
 #[derive(Debug)]
 pub struct StepEffectAndUpcomingEraValidators {
-    /// An [`ExecutionEffect`] created by an era ending.
-    pub step_execution_effect: ExecutionEffect,
     /// Validator sets for all upcoming eras that have already been determined.
     pub upcoming_era_validators: BTreeMap<EraId, BTreeMap<PublicKey, U512>>,
+    /// An [`ExecutionJournal`] created by an era ending.
+    pub step_execution_journal: ExecutionJournal,
 }
 
 /// A [`Block`] that was the result of execution in the `ContractRuntime` along with any execution
@@ -98,6 +98,6 @@ pub struct BlockAndExecutionEffects {
     pub block: Block,
     /// The results from executing the deploys in the block.
     pub execution_results: HashMap<DeployHash, (DeployHeader, ExecutionResult)>,
-    /// The [`ExecutionEffect`] and the upcoming validator sets determined by the `step`
+    /// The [`ExecutionJournal`] and the upcoming validator sets determined by the `step`
     pub maybe_step_effect_and_upcoming_era_validators: Option<StepEffectAndUpcomingEraValidators>,
 }
