@@ -42,7 +42,7 @@ use std::{
     collections::{BTreeMap, HashMap, HashSet},
     convert::Infallible,
     fmt::{self, Debug, Display, Formatter},
-    io, mem,
+    io,
     net::{SocketAddr, TcpListener},
     result,
     sync::{Arc, Weak},
@@ -55,7 +55,7 @@ use futures::{future::BoxFuture, FutureExt};
 use openssl::{error::ErrorStack as OpenSslErrorStack, pkey};
 use pkey::{PKey, Private};
 use prometheus::Registry;
-use rand::seq::IteratorRandom;
+use rand::seq::{IteratorRandom, SliceRandom};
 use serde::{Deserialize, Serialize};
 use thiserror::Error;
 use tokio::{
@@ -84,17 +84,14 @@ use self::{
 pub(crate) use self::{
     event::Event,
     gossiped_address::GossipedAddress,
-    message::{Message, MessageKind, Payload},
+    message::{Message, MessageKind, Payload, PayloadWeights},
 };
-use super::consensus;
+use super::{consensus, contract_runtime::ContractRuntimeAnnouncement};
 use crate::{
     components::{networking_metrics::NetworkingMetrics, Component},
     effect::{
-        announcements::{BlocklistAnnouncement, LinearChainAnnouncement, NetworkAnnouncement},
-        requests::{
-            ChainspecLoaderRequest, ContractRuntimeRequest, NetworkInfoRequest, NetworkRequest,
-            StorageRequest,
-        },
+        announcements::{BlocklistAnnouncement, NetworkAnnouncement},
+        requests::{NetworkInfoRequest, NetworkRequest, StorageRequest},
         EffectBuilder, EffectExt, Effects,
     },
     reactor::{EventQueueHandle, Finalize, ReactorEvent},
@@ -104,10 +101,8 @@ use crate::{
     NodeRng,
 };
 use chain_info::ChainInfo;
-pub use config::Config;
-pub use error::Error;
-
-const MAX_ASYMMETRIC_TIME: Duration = Duration::from_secs(60);
+pub(crate) use config::Config;
+pub(crate) use error::Error;
 
 const MAX_METRICS_DROP_ATTEMPTS: usize = 25;
 const DROP_RETRY_DELAY: Duration = Duration::from_millis(100);
@@ -127,11 +122,8 @@ const BASE_RECONNECTION_TIMEOUT: Duration = Duration::from_secs(1);
 /// Interval during which to perform outgoing manager housekeeping.
 const OUTGOING_MANAGER_SWEEP_INTERVAL: Duration = Duration::from_secs(1);
 
-/// Interval for checking for symmetrical connections.
-const SYMMETRY_SWEEP_INTERVAL: Duration = Duration::from_secs(30);
-
 #[derive(Clone, DataSize, Debug)]
-pub struct OutgoingHandle<P> {
+pub(crate) struct OutgoingHandle<P> {
     #[data_size(skip)] // Unfortunately, there is no way to inspect an `UnboundedSender`.
     sender: UnboundedSender<Arc<Message<P>>>,
     peer_addr: SocketAddr,
@@ -176,12 +168,6 @@ where
     #[data_size(skip)]
     net_metrics: Arc<NetworkingMetrics>,
 
-    /// The highest era seen so far.
-    ///
-    /// The era supervisor currently does not allow for easy access to the concept of the "current"
-    /// era, so we treat the highest era we have seen as the active era.
-    highest_era_seen: EraId,
-
     /// The outgoing bandwidth limiter.
     #[data_size(skip)]
     outgoing_limiter: Box<dyn Limiter>,
@@ -191,17 +177,16 @@ where
     /// This is not incoming bandwidth but an independent resource estimate.
     #[data_size(skip)]
     incoming_limiter: Box<dyn Limiter>,
+
+    /// The era that is considered the active era by the small network component.
+    active_era: EraId,
 }
 
 impl<REv, P> SmallNetwork<REv, P>
 where
     P: Payload + 'static,
-    REv: ReactorEvent
-        + From<Event<P>>
-        + From<NetworkAnnouncement<NodeId, P>>
-        + From<ContractRuntimeRequest>
-        + From<StorageRequest>
-        + From<ChainspecLoaderRequest>,
+    REv:
+        ReactorEvent + From<Event<P>> + From<NetworkAnnouncement<NodeId, P>> + From<StorageRequest>,
 {
     /// Creates a new small network component instance.
     #[allow(clippy::type_complexity)]
@@ -212,7 +197,6 @@ where
         registry: &Registry,
         small_network_identity: SmallNetworkIdentity,
         chain_info_source: C,
-        initial_era: Option<EraId>,
     ) -> Result<(SmallNetwork<REv, P>, Effects<Event<P>>)> {
         let mut known_addresses = HashSet::new();
         for address in &cfg.known_addresses {
@@ -298,6 +282,7 @@ where
             chain_info: chain_info_source.into(),
             public_addr,
             consensus_keys,
+            payload_weights: cfg.estimator_weights.clone(),
         });
 
         // Run the server task.
@@ -307,11 +292,14 @@ where
 
         let (server_shutdown_sender, server_shutdown_receiver) = watch::channel(());
         let shutdown_receiver = server_shutdown_receiver.clone();
-        let server_join_handle = tokio::spawn(tasks::server(
-            context.clone(),
-            tokio::net::TcpListener::from_std(listener).map_err(Error::ListenerConversion)?,
-            server_shutdown_receiver,
-        ));
+        let server_join_handle = tokio::spawn(
+            tasks::server(
+                context.clone(),
+                tokio::net::TcpListener::from_std(listener).map_err(Error::ListenerConversion)?,
+                server_shutdown_receiver,
+            )
+            .in_current_span(),
+        );
 
         let mut component = SmallNetwork {
             cfg,
@@ -322,9 +310,10 @@ where
             shutdown_receiver,
             server_join_handle: Some(server_join_handle),
             net_metrics,
-            highest_era_seen: EraId::new(0),
             outgoing_limiter,
             incoming_limiter,
+            // We start with an empty set of validators for era 0 and expect to be updated.
+            active_era: EraId::new(0),
         };
 
         let effect_builder = EffectBuilder::new(event_queue);
@@ -336,12 +325,7 @@ where
             .filter_map(|addr| component.outgoing_manager.learn_addr(addr, true, now))
             .collect();
 
-        // Initialize the known validator set with the active era, if given.
-        let mut effects = initial_era
-            .map(|era_id| component.handle_active_era_change(effect_builder, era_id))
-            .unwrap_or_default();
-
-        effects.extend(component.process_dial_requests(dial_requests));
+        let mut effects = component.process_dial_requests(dial_requests);
 
         // Start broadcasting our public listening address.
         effects.extend(
@@ -459,7 +443,7 @@ where
                 peer_consensus_public_key,
                 stream,
             } => {
-                info!("new incoming connection established");
+                info!(%public_addr, "new incoming connection established");
 
                 // Learn the address the peer gave us.
                 let dial_requests =
@@ -608,7 +592,7 @@ where
                 info!("new outgoing connection established");
 
                 let (sender, receiver) = mpsc::unbounded_channel();
-                let handle = OutgoingHandle { peer_addr, sender };
+                let handle = OutgoingHandle { sender, peer_addr };
 
                 let request = self
                     .outgoing_manager
@@ -675,36 +659,6 @@ where
             .ignore()
     }
 
-    /// Sweeps across connection symmetry, enforcing symmetrical connections.
-    fn enforce_symmetric_connections(&mut self, now: Instant) -> Effects<Event<P>> {
-        let mut dial_requests = Vec::new();
-
-        let conn_syms = mem::take(&mut self.connection_symmetries);
-        self.connection_symmetries = conn_syms
-            .into_iter()
-            .filter_map(|(peer_id, sym)| {
-                if sym.should_be_reaped(now, MAX_ASYMMETRIC_TIME) {
-                    info!(%peer_id, "reaping asymmetric connection");
-
-                    // Get the outgoing connection and block it.
-                    if let Some(addr) = self.outgoing_manager.get_addr(peer_id) {
-                            if let Some(req) = self.outgoing_manager.block_addr(addr, now) {
-                                dial_requests.push(req);
-                            }
-                    } else {
-                        debug!(%peer_id, "tried to reap non-existent asymmetric connection, must be incoming only");
-                    }
-
-                    None
-                } else {
-                    Some((peer_id, sym))
-                }
-            })
-            .collect();
-
-        self.process_dial_requests(dial_requests)
-    }
-
     /// Processes a set of `DialRequest`s, updating the component and emitting needed effects.
     fn process_dial_requests<T>(&mut self, requests: T) -> Effects<Event<P>>
     where
@@ -760,61 +714,6 @@ where
         })
     }
 
-    /// Handle the change of the active era.
-    fn handle_active_era_change(
-        &mut self,
-        effect_builder: EffectBuilder<REv>,
-        era_id: EraId,
-    ) -> Effects<Event<P>>
-    where
-        REv: From<ContractRuntimeRequest> + From<StorageRequest> + From<ChainspecLoaderRequest>,
-    {
-        // The current era is `block.era`, but it will only proceed for another block.
-        //
-        // `current`: Era that is just about to end, `block.era`.
-        // `next`: Era that will begin shortly.
-        // `upcoming`: Era after `next`.
-
-        async move {
-            let current: HashSet<PublicKey> = effect_builder
-                .get_era_validators(era_id)
-                .await
-                .map(|validators| validators.into_iter().map(|(k, _)| k).collect())
-                .unwrap_or_else(|| {
-                    warn!("could not determine current era validators");
-                    Default::default()
-                });
-            let next: HashSet<PublicKey> = effect_builder
-                .get_era_validators(era_id.successor())
-                .await
-                .map(|validators| validators.into_iter().map(|(k, _)| k).collect())
-                .unwrap_or_else(|| {
-                    warn!("could not determine next era validators");
-                    Default::default()
-                });
-            let upcoming_validators: HashSet<PublicKey> = effect_builder
-                .get_era_validators(era_id.successor().successor())
-                .await
-                .map(|validators| validators.into_iter().map(|(k, _)| k).collect())
-                .unwrap_or_else(|| {
-                    debug!("could not determine upcoming (current+2) era validators");
-                    Default::default()
-                });
-
-            let mut active_validators: HashSet<PublicKey> = HashSet::new();
-            active_validators.extend(current.into_iter());
-            active_validators.extend(next.into_iter());
-
-            (Box::new(active_validators), Box::new(upcoming_validators))
-        }
-        .event(
-            |(active_validators, upcoming_validators)| Event::ValidatorsChanged {
-                active_validators,
-                upcoming_validators,
-            },
-        )
-    }
-
     /// Emits an announcement that a connection has been completed.
     fn connection_completed(
         &self,
@@ -885,12 +784,8 @@ where
 
 impl<REv, P> Component<REv> for SmallNetwork<REv, P>
 where
-    REv: ReactorEvent
-        + From<Event<P>>
-        + From<NetworkAnnouncement<NodeId, P>>
-        + From<ContractRuntimeRequest>
-        + From<StorageRequest>
-        + From<ChainspecLoaderRequest>,
+    REv:
+        ReactorEvent + From<Event<P>> + From<NetworkAnnouncement<NodeId, P>> + From<StorageRequest>,
     P: Payload,
 {
     type Event = Event<P>;
@@ -963,6 +858,11 @@ where
                 NetworkInfoRequest::GetPeers { responder } => {
                     responder.respond(self.peers()).ignore()
                 }
+                NetworkInfoRequest::GetPeersInRandomOrder { responder } => {
+                    let mut peers_vec: Vec<NodeId> = self.peers().keys().cloned().collect();
+                    peers_vec.shuffle(rng);
+                    responder.respond(peers_vec).ignore()
+                }
             },
             Event::PeerAddressReceived(gossiped_address) => {
                 let requests = self.outgoing_manager.learn_addr(
@@ -985,30 +885,63 @@ where
                     Effects::new()
                 }
             }
+            Event::ContractRuntimeAnnouncement(
+                ContractRuntimeAnnouncement::LinearChainBlock(_)
+                | ContractRuntimeAnnouncement::StepSuccess { .. },
+            ) => Effects::new(),
+            Event::ContractRuntimeAnnouncement(
+                ContractRuntimeAnnouncement::UpcomingEraValidators {
+                    era_that_is_ending,
+                    mut upcoming_era_validators,
+                },
+            ) => {
+                if era_that_is_ending < self.active_era {
+                    debug!("ignoring past era end announcement");
+                } else {
+                    // We have a new `active_era`, even if we may have skipped some, as this one
+                    // is the highest seen.
+                    self.active_era = era_that_is_ending + 1;
+
+                    let active_validators: HashSet<PublicKey> = upcoming_era_validators
+                        .remove(&self.active_era)
+                        .unwrap_or_default()
+                        .into_keys()
+                        .collect();
+
+                    if active_validators.is_empty() {
+                        error!("received an empty set of active era validators");
+                    }
+
+                    let upcoming_validators: HashSet<PublicKey> = upcoming_era_validators
+                        .remove(&(self.active_era + 1))
+                        .unwrap_or_default()
+                        .into_keys()
+                        .collect();
+
+                    debug!(
+                        %era_that_is_ending,
+                        active = active_validators.len(),
+                        upcoming = upcoming_validators.len(),
+                        "updating active and upcoming validators"
+                    );
+                    self.incoming_limiter
+                        .update_validators(active_validators.clone(), upcoming_validators.clone());
+                    self.outgoing_limiter
+                        .update_validators(active_validators, upcoming_validators);
+                }
+
+                Effects::new()
+            }
 
             Event::GossipOurAddress => {
                 let mut effects = self.gossip_our_address(effect_builder);
                 effects.extend(
                     effect_builder
-                        .set_timeout(self.cfg.gossip_interval)
+                        .set_timeout(self.cfg.gossip_interval.into())
                         .event(|_| Event::GossipOurAddress),
                 );
                 effects
             }
-
-            Event::SweepSymmetries => {
-                let now = Instant::now();
-
-                let mut effects = self.enforce_symmetric_connections(now);
-
-                effects.extend(
-                    effect_builder
-                        .set_timeout(SYMMETRY_SWEEP_INTERVAL)
-                        .event(|_| Event::SweepSymmetries),
-                );
-
-                effects
-            }
             Event::SweepOutgoing => {
                 let now = Instant::now();
                 let requests = self.outgoing_manager.perform_housekeeping(now);
@@ -1022,45 +955,12 @@ where
 
                 effects
             }
-            Event::LinearChainAnnouncement(LinearChainAnnouncement::BlockAdded(block)) => {
-                // On switch blocks, we need to update our validator sets.
-                if block.header().is_switch_block() {
-                    let era_id = block.header().era_id();
-
-                    if era_id > self.highest_era_seen {
-                        self.highest_era_seen = era_id;
-                        self.handle_active_era_change(effect_builder, era_id)
-                    } else {
-                        debug!(highest_era_seen=%self.highest_era_seen, %era_id,
-                               "ignoring era, as it is not the highest seen");
-                        Effects::new()
-                    }
-                } else {
-                    trace!("ignoring non-switch block");
-                    Effects::new()
-                }
-            }
-            Event::LinearChainAnnouncement(LinearChainAnnouncement::NewFinalitySignature(_)) => {
-                Effects::new()
-            }
-            Event::ValidatorsChanged {
-                active_validators,
-                upcoming_validators,
-            } => {
-                self.outgoing_limiter.update_validators(
-                    (*active_validators).clone(),
-                    (*upcoming_validators).clone(),
-                );
-                self.incoming_limiter
-                    .update_validators(*active_validators, *upcoming_validators);
-                Effects::new()
-            }
         }
     }
 }
 
 #[derive(Debug, Error)]
-pub enum SmallNetworkIdentityError {
+pub(crate) enum SmallNetworkIdentityError {
     #[error("could not generate TLS certificate: {0}")]
     CouldNotGenerateTlsCertificate(OpenSslErrorStack),
     #[error(transparent)]
@@ -1069,13 +969,13 @@ pub enum SmallNetworkIdentityError {
 
 /// An ephemeral [PKey<Private>] and [TlsCert] that identifies this node
 #[derive(DataSize, Debug, Clone)]
-pub struct SmallNetworkIdentity {
+pub(crate) struct SmallNetworkIdentity {
     secret_key: Arc<PKey<Private>>,
     tls_certificate: Arc<TlsCert>,
 }
 
 impl SmallNetworkIdentity {
-    pub fn new() -> result::Result<Self, SmallNetworkIdentityError> {
+    pub(crate) fn new() -> result::Result<Self, SmallNetworkIdentityError> {
         let (not_yet_validated_x509_cert, secret_key) = tls::generate_node_cert()
             .map_err(SmallNetworkIdentityError::CouldNotGenerateTlsCertificate)?;
         let tls_certificate = tls::validate_cert(not_yet_validated_x509_cert)?;
@@ -1112,7 +1012,7 @@ impl From<&SmallNetworkIdentity> for NodeId {
 type Transport = SslStream<TcpStream>;
 
 /// A framed transport for `Message`s.
-pub type FramedTransport<P> = tokio_serde::Framed<
+pub(crate) type FramedTransport<P> = tokio_serde::Framed<
     tokio_util::codec::Framed<Transport, LengthDelimitedCodec>,
     Message<P>,
     Arc<Message<P>>,
