@@ -147,7 +147,7 @@ pub(crate) enum Event {
 
 /// A storage component error.
 #[derive(Debug, Error)]
-pub(crate) enum Error {
+pub enum Error {
     /// Failure to create the root database directory.
     #[error("failed to create database directory `{}`: {}", .0.display(), .1)]
     CreateDatabaseDirectory(PathBuf, io::Error),
@@ -306,8 +306,9 @@ impl From<lmdb::Error> for Error {
     }
 }
 
+/// Storage component.
 #[derive(DataSize, Debug)]
-pub(crate) struct Storage {
+pub struct Storage {
     /// Storage location.
     root: PathBuf,
     /// Environment holding LMDB databases.
@@ -394,7 +395,7 @@ impl Storage {
     /// If `should_check_integrity` is true, time-consuming integrity checks will be performed
     /// during this call to `new()`, potentially blocking for several minutes.  This should normally
     /// only be required if the node is detected to have restarted after a crash.
-    pub(crate) fn new(
+    pub fn new(
         cfg: &WithDir<Config>,
         hard_reset_to_start_of_era: Option<EraId>,
         protocol_version: ProtocolVersion,
@@ -602,14 +603,8 @@ impl Storage {
                 txn.commit()?;
                 Ok(responder.respond(()).ignore())
             }
-            #[cfg(test)]
             StateStoreRequest::Load { key, responder } => {
-                let txn = self.env.begin_ro_txn()?;
-                let bytes = match txn.get(self.state_store_db, &key) {
-                    Ok(slice) => Some(slice.to_owned()),
-                    Err(lmdb::Error::NotFound) => None,
-                    Err(err) => return Err(err.into()),
-                };
+                let bytes = self.read_state_store(&key)?;
                 Ok(responder.respond(bytes).ignore())
             }
         }
@@ -708,10 +703,7 @@ impl Storage {
                 .respond(self.get_transfers(&mut self.env.begin_ro_txn()?, &block_hash)?)
                 .ignore(),
             StorageRequest::PutDeploy { deploy, responder } => {
-                let mut txn = self.env.begin_rw_txn()?;
-                let outcome = txn.put_value(self.deploy_db, deploy.id(), &deploy, false)?;
-                txn.commit()?;
-                responder.respond(outcome).ignore()
+                responder.respond(self.put_deploy(&*deploy)?).ignore()
             }
             StorageRequest::GetDeploys {
                 deploy_hashes,
@@ -901,6 +893,14 @@ impl Storage {
         })
     }
 
+    /// Put a single deploy into storage.
+    pub fn put_deploy(&self, deploy: &Deploy) -> Result<bool, Error> {
+        let mut txn = self.env.begin_rw_txn()?;
+        let outcome = txn.put_value(self.deploy_db, deploy.id(), &deploy, false)?;
+        txn.commit()?;
+        Ok(outcome)
+    }
+
     /// Retrieves single block header by height by looking it up in the index and returning it.
     fn get_block_header_and_metadata_by_height<Tx: Transaction>(
         &self,
@@ -926,14 +926,20 @@ impl Storage {
     }
 
     /// Retrieves a block by hash.
-    fn read_block(&self, block_hash: &BlockHash) -> Result<Option<Block>, Error> {
+    pub fn read_block(&self, block_hash: &BlockHash) -> Result<Option<Block>, Error> {
         self.get_single_block(&mut self.env.begin_ro_txn()?, block_hash)
     }
 
+    /// Directly returns a deploy from internal store.
+    pub fn read_deploy_by_hash(&self, deploy_hash: DeployHash) -> Result<Option<Deploy>, Error> {
+        let mut txn = self.env.begin_ro_txn()?;
+        Ok(txn.get_value(self.deploy_db, &deploy_hash)?)
+    }
+
     /// Writes a block to storage, updating indices as necessary
     /// Returns `Ok(true)` if the block has been successfully written, `Ok(false)` if a part of it
     /// couldn't be written because it already existed, and `Err(_)` if there was an error.
-    fn write_block(&mut self, block: &Block) -> Result<bool, Error> {
+    pub fn write_block(&mut self, block: &Block) -> Result<bool, Error> {
         // Validate the block prior to inserting it into the database
         block.verify()?;
         let mut txn = self.env.begin_rw_txn()?;
@@ -999,6 +1005,16 @@ impl Storage {
             .transpose()
     }
 
+    /// Retrieves single block by height by looking it up in the index and returning it.
+    pub fn read_block_by_height(&self, height: u64) -> Result<Option<Block>, Error> {
+        self.get_block_by_height(&mut self.env.begin_ro_txn()?, height)
+    }
+
+    /// Gets the highest block.
+    pub fn read_highest_block(&self) -> Result<Option<Block>, Error> {
+        self.get_highest_block(&mut self.env.begin_ro_txn()?)
+    }
+
     /// Retrieves single block by height by looking it up in the index and returning it.
     fn get_block_by_height<Tx: Transaction>(
         &self,
@@ -1348,6 +1364,18 @@ impl Storage {
     ) -> Result<Option<BlockSignatures>, Error> {
         Ok(tx.get_value(self.block_metadata_db, block_hash)?)
     }
+
+    /// Retrieves single switch block by era ID by looking it up in the index and returning it.
+    fn get_switch_block_by_era_id<Tx: Transaction>(
+        &self,
+        tx: &mut Tx,
+        era_id: EraId,
+    ) -> Result<Option<Block>, Error> {
+        self.switch_block_era_id_index
+            .get(&era_id)
+            .and_then(|block_hash| self.get_single_block(tx, block_hash).transpose())
+            .transpose()
+    }
 }
 
 /// Inserts the relevant entries to the two indices.
@@ -1486,11 +1514,11 @@ fn move_storage_files_to_network_subdir(
 /// On-disk storage configuration.
 #[derive(Clone, DataSize, Debug, Deserialize, Serialize)]
 #[serde(deny_unknown_fields)]
-pub(crate) struct Config {
+pub struct Config {
     /// The path to the folder where any files created or read by the storage component will exist.
     ///
     /// If the folder doesn't exist, it and any required parents will be created.
-    pub(crate) path: PathBuf,
+    pub path: PathBuf,
     /// The maximum size of the database to use for the block store.
     ///
     /// The size should be a multiple of the OS page size.
@@ -1611,6 +1639,31 @@ impl Storage {
             }
         }
     }
+
+    /// Get the switch block for a specified era number in a read-only LMDB database transaction.
+    ///
+    /// # Panics
+    ///
+    /// Panics on any IO or db corruption error.
+    pub fn transactional_get_switch_block_by_era_id(
+        &self,
+        switch_block_era_num: u64,
+    ) -> Option<Block> {
+        let mut read_only_lmdb_transaction = self
+            .env
+            .begin_ro_txn()
+            .expect("Could not start read only transaction for lmdb");
+        let switch_block = self
+            .get_switch_block_by_era_id(
+                &mut read_only_lmdb_transaction,
+                EraId::from(switch_block_era_num),
+            )
+            .expect("LMDB panicked trying to get switch block");
+        read_only_lmdb_transaction
+            .commit()
+            .expect("Could not commit transaction");
+        switch_block
+    }
 }
 
 // Testing code. The functions below allow direct inspection of the storage component and should
@@ -1653,43 +1706,6 @@ impl Storage {
             })
             .collect()
     }
-
-    /// Retrieves single switch block by era ID by looking it up in the index and returning it.
-    fn get_switch_block_by_era_id<Tx: Transaction>(
-        &self,
-        tx: &mut Tx,
-        era_id: EraId,
-    ) -> Result<Option<Block>, Error> {
-        self.switch_block_era_id_index
-            .get(&era_id)
-            .and_then(|block_hash| self.get_single_block(tx, block_hash).transpose())
-            .transpose()
-    }
-
-    /// Get the switch block for a specified era number in a read-only LMDB database transaction.
-    ///
-    /// # Panics
-    ///
-    /// Panics on any IO or db corruption error.
-    pub(crate) fn transactional_get_switch_block_by_era_id(
-        &self,
-        switch_block_era_num: u64,
-    ) -> Option<Block> {
-        let mut read_only_lmdb_transaction = self
-            .env
-            .begin_ro_txn()
-            .expect("Could not start read only transaction for lmdb");
-        let switch_block = self
-            .get_switch_block_by_era_id(
-                &mut read_only_lmdb_transaction,
-                EraId::from(switch_block_era_num),
-            )
-            .expect("LMDB panicked trying to get switch block");
-        read_only_lmdb_transaction
-            .commit()
-            .expect("Could not commit transaction");
-        switch_block
-    }
 }
 
 fn construct_block_body_to_block_header_reverse_lookup(
@@ -2037,30 +2053,33 @@ fn initialize_deploy_metadata_db(
     deleted_block_hashes: &HashSet<BlockHash>,
 ) -> Result<(), LmdbExtError> {
     info!("initializing deploy metadata database");
-    let mut txn = env.begin_rw_txn()?;
-    let mut cursor = txn.open_rw_cursor(*deploy_metadata_db)?;
 
-    for (raw_key, raw_val) in cursor.iter() {
-        let mut deploy_metadata: DeployMetadata = lmdb_ext::deserialize(raw_val)?;
-        let len_before = deploy_metadata.execution_results.len();
+    if !deleted_block_hashes.is_empty() {
+        let mut txn = env.begin_rw_txn()?;
+        let mut cursor = txn.open_rw_cursor(*deploy_metadata_db)?;
 
-        deploy_metadata.execution_results = deploy_metadata
-            .execution_results
-            .drain()
-            .filter(|(block_hash, _)| !deleted_block_hashes.contains(block_hash))
-            .collect();
+        for (raw_key, raw_val) in cursor.iter() {
+            let mut deploy_metadata: DeployMetadata = lmdb_ext::deserialize(raw_val)?;
+            let len_before = deploy_metadata.execution_results.len();
 
-        // If the deploy's execution results are now empty, we just remove them entirely.
-        if deploy_metadata.execution_results.is_empty() {
-            cursor.del(WriteFlags::empty())?;
-        } else if len_before != deploy_metadata.execution_results.len() {
-            let buffer = lmdb_ext::serialize(&deploy_metadata)?;
-            cursor.put(&raw_key, &buffer, WriteFlags::empty())?;
+            deploy_metadata.execution_results = deploy_metadata
+                .execution_results
+                .drain()
+                .filter(|(block_hash, _)| !deleted_block_hashes.contains(block_hash))
+                .collect();
+
+            // If the deploy's execution results are now empty, we just remove them entirely.
+            if deploy_metadata.execution_results.is_empty() {
+                cursor.del(WriteFlags::empty())?;
+            } else if len_before != deploy_metadata.execution_results.len() {
+                let buffer = lmdb_ext::serialize(&deploy_metadata)?;
+                cursor.put(&raw_key, &buffer, WriteFlags::empty())?;
+            }
         }
-    }
 
-    drop(cursor);
-    txn.commit()?;
+        drop(cursor);
+        txn.commit()?;
+    }
 
     info!("deploy metadata database initialized");
     Ok(())
