@@ -8,7 +8,7 @@
 //! While the [`main`](fn.main.html) function is the central entrypoint for the node application,
 //! its core event loop is found inside the [reactor](reactor/index.html).
 
-#![doc(html_root_url = "https://docs.rs/casper-node/1.4.1")]
+#![doc(html_root_url = "https://docs.rs/casper-node/1.4.2")]
 #![doc(
     html_favicon_url = "https://raw.githubusercontent.com/CasperLabs/casper-node/master/images/CasperLabs_Logo_Favicon_RGB_50px.png",
     html_logo_url = "https://raw.githubusercontent.com/CasperLabs/casper-node/master/images/CasperLabs_Logo_Symbol_RGB.png",
@@ -37,7 +37,13 @@ pub(crate) mod utils;
 pub mod cli;
 pub mod crypto;
 pub mod types;
-pub use components::rpc_server::rpcs;
+
+pub use components::{
+    contract_runtime,
+    rpc_server::rpcs,
+    storage::{self, Config as StorageConfig},
+};
+pub use utils::WithDir;
 
 use std::sync::{
     atomic::{AtomicBool, AtomicUsize},
@@ -60,8 +66,9 @@ pub(crate) use components::{
     event_stream_server::Config as EventStreamServerConfig, fetcher::Config as FetcherConfig,
     gossiper::Config as GossipConfig, linear_chain_sync::Config as LinearChainSyncConfig,
     rest_server::Config as RestServerConfig, rpc_server::Config as RpcServerConfig,
-    small_network::Config as SmallNetworkConfig, storage::Config as StorageConfig,
+    small_network::Config as SmallNetworkConfig,
 };
+
 pub(crate) use types::NodeRng;
 
 /// The maximum thread count which should be spawned by the tokio runtime.
