@@ -3,11 +3,7 @@ use datasize::DataSize;
 use rand::Rng;
 use serde::Serialize;
 
-use casper_execution_engine::shared::motes::Motes;
-use casper_types::{
-    bytesrepr::{self, FromBytes, ToBytes},
-    PublicKey,
-};
+use casper_types::bytesrepr::{self, FromBytes, ToBytes};
 
 use super::AccountsConfig;
 #[cfg(test)]
@@ -25,24 +21,6 @@ pub struct NetworkConfig {
     pub(crate) accounts_config: AccountsConfig,
 }
 
-impl NetworkConfig {
-    #[allow(unused)]
-    /// Returns a vector of chainspec validators' public key and their stake.
-    pub fn chainspec_validator_stakes(&self) -> Vec<(PublicKey, Motes)> {
-        self.accounts_config
-            .accounts()
-            .iter()
-            .filter_map(|account_config| {
-                if account_config.is_genesis_validator() {
-                    Some((account_config.public_key(), account_config.bonded_amount()))
-                } else {
-                    None
-                }
-            })
-            .collect()
-    }
-}
-
 #[cfg(test)]
 impl NetworkConfig {
     /// Generates a random instance using a `TestRng`.
@@ -55,8 +33,8 @@ impl NetworkConfig {
 
         NetworkConfig {
             name,
-            accounts_config,
             maximum_net_message_size,
+            accounts_config,
         }
     }
 }
@@ -84,8 +62,8 @@ impl FromBytes for NetworkConfig {
         let (maximum_net_message_size, remainder) = FromBytes::from_bytes(remainder)?;
         let config = NetworkConfig {
             name,
-            accounts_config,
             maximum_net_message_size,
+            accounts_config,
         };
         Ok((config, remainder))
     }
